clear all
tic

dim = 4;
nobs = 2000;
nMC = 100;
startMC = 1;
df = 6;

% Preallocate memory
Y_mat = zeros(nobs,dim);
censor_ratio = zeros(dim+1,nMC);
censor_regime = zeros(2^dim,nMC);
beta_ML = zeros(dim*3,nMC);
beta_EM = zeros(dim*3,nMC);
beta_BAY = zeros(dim*3,nMC);
sigma_ML = zeros(dim*(dim+1)/2,nMC);
sigma_EM = zeros(dim*(dim+1)/2,nMC);
sigma_BAY = zeros(dim*(dim+1)/2,nMC);
df_ML = zeros(1,nMC);
df_EM = zeros(1,nMC);
df_BAY = zeros(1,nMC);
time_ML = zeros(1,nMC);
time_EM = zeros(1,nMC);
time_BAY = zeros(1,nMC);

% Simulation begins
H_status_bar = waitbar(0,'Simulation  ');
for r = startMC:nMC
    
    waitbar(r/nMC,H_status_bar,['Simulation   ',num2str(r)]);
    
    % Generating data
    rand('seed',r*12345);
    randn('seed',r*1234567);
    sigma = 0.3*ones(dim,dim) + 0.7*eye(dim);
    epsilon = mvtrnd(sigma,df,nobs);
    for m = 1:dim
        X = [ones(nobs,1),randn(nobs,1),randn(nobs,1)];        
        Y = X * [m;m;m] + epsilon(:,m);
        Y(Y<0) = 0;
        Y_mat(:,m) = Y;
        eval(['X',num2str(m),'= X;'])
        eval(['Y',num2str(m),'= Y;'])
    end
    
    % summary of censoring 
    censor_index = Y_mat' == 0;
    censor_cum = sum( censor_index .* repmat(2.^(0:dim-1)',1,nobs) );
    censor_regime(:,r) = mean( censor_cum(ones(2^dim,1),:) == repmat((0:2^dim-1)',1,nobs) ,2);
         
    ncensor = sum(censor_index)';
    seq = 0:dim;
    censor_ratio(:,r) = mean( ncensor(:,ones(dim+1,1)) == seq(ones(nobs,1),:) )';
    
    % Estimation via ML,EM,Bayesian
    if dim == 4
        try            
            tic
            waitbar(r/nMC,H_status_bar,['Simulation   ',num2str(r),' : ML in progress']);
            [beta_ML(:,r),sigma_temp,df_ML(r)] = ML_CT_SUR(Y1,X1,Y2,X2,Y3,X3,Y4,X4);
            sigma_ML(:,r) = vech(sigma_temp);
            time_ML(r) = toc;
        catch
            disp(['Simulation ',num2str(r),': an error occurs in ML'])
        end
        
        try
            tic
            waitbar(r/nMC,H_status_bar,['Simulation   ',num2str(r),' : EM in progress']);
            [beta_EM(:,r),sigma_temp,df_EM(r)] = EM_CT_SUR(Y1,X1,Y2,X2,Y3,X3,Y4,X4);
            sigma_EM(:,r) = vech(sigma_temp);
            time_EM(r) = toc;
        catch
            disp(['Simulation ',num2str(r),': an error occurs in EM '])
        end
        
        try
            tic
            waitbar(r/nMC,H_status_bar,['Simulation   ',num2str(r),' : Bayesian in progress']);
            [beta_BAY(:,r),sigma_temp,df_BAY(r)] = GIBBS_CT_SUR(Y1,X1,Y2,X2,Y3,X3,Y4,X4);
            sigma_BAY(:,r) = vech(sigma_temp);
            time_BAY(r) = toc;
        catch
            disp(['Simulation ',num2str(r),': an error occurs in Bayesian MCMC '])
        end
        
    else
        error('Please modify the inputs: add/delete some Yi, Xi')
    end
    
end
close(H_status_bar);


%----Simulation Done!--------------

% summary of censoring
censor_regime_avg = mean(censor_regime,2);
censor_regime_avg = censor_regime_avg([1,2,3,5,9,4,6,10,7,11,13,8,12,14,15,16]);
censor_ratio_avg = mean(censor_ratio,2);

% calculate statistics
para_ML = [beta_ML;nan(1,nMC);sigma_ML;nan(1,nMC);df_ML];
para_EM = [beta_EM;nan(1,nMC);sigma_EM;nan(1,nMC);df_EM];
para_BAY = [beta_BAY;nan(1,nMC);sigma_BAY;nan(1,nMC);df_BAY];

para_ML_mean = mean(para_ML,2);
para_EM_mean = mean(para_EM,2);
para_BAY_mean = mean(para_BAY,2);

para_ML_std = std(para_ML,0,2);
para_EM_std = std(para_EM,0,2);
para_BAY_std = std(para_BAY,0,2);

para_true = [reshape( repmat(1:dim,3,1), dim*3,1);nan;vech(sigma);nan;df];
para_ML_MSE =  sum( (para_ML - para_true(:,ones(nMC,1))).^2 ,2) / nMC;
para_EM_MSE =  sum( (para_EM - para_true(:,ones(nMC,1))).^2 ,2) / nMC;
para_BAY_MSE =  sum( (para_BAY - para_true(:,ones(nMC,1))).^2 ,2) / nMC;

stat_pool = [para_ML_mean,para_ML_std,para_ML_MSE,...
    para_EM_mean,para_EM_std,para_EM_MSE,...
    para_BAY_mean,para_BAY_std,para_BAY_MSE];

disp('Individual statistics')

npara = dim*3 + dim*(dim+1)/2 + 1;

disp('ML  ML_STD  ML_MSE  EM  EM_STD  EM_MSE  BAY  BAY_STD  BAY_MSE')
for m=1:npara+2
    fprintf('%8.2f', stat_pool(m,:))
    fprintf('\n')
end
fprintf('\n')


% compare estimators
%--------------
% Compare beta
%--------------
beta_true = reshape( repmat(1:dim,3,1), dim*3,1);
beta_diff_ML = beta_ML - beta_true(:,ones(nMC,1));
MSE_beta_ML = beta_diff_ML * beta_diff_ML' / nMC;
QUAD_beta_ML = mean( sum (beta_diff_ML.^2) );

beta_diff_EM = beta_EM - beta_true(:,ones(nMC,1));
MSE_beta_EM = beta_diff_EM * beta_diff_EM' / nMC;
QUAD_beta_EM = mean( sum (beta_diff_EM.^2) );

beta_diff_BAY = beta_BAY - beta_true(:,ones(nMC,1));
MSE_beta_BAY = beta_diff_BAY * beta_diff_BAY' / nMC;
QUAD_beta_BAY = mean( sum (beta_diff_BAY.^2) );

disp(' ')
disp(['Quadric risk of  ML :  ',num2str(QUAD_beta_ML)])
disp(['Quadric risk of  EM :  ',num2str(QUAD_beta_EM)])
disp(['Quadric risk of  BAY:  ',num2str(QUAD_beta_BAY)])

eig_ML_EM = eig(MSE_beta_ML - MSE_beta_EM);
eig_ML_BAY = eig(MSE_beta_ML - MSE_beta_BAY);
eig_EM_BAY = eig(MSE_beta_EM - MSE_beta_BAY);

if all( eig_ML_EM < 0 )
    disp('ML is preferable to EM in terms of  MSE definiteness')
elseif all( eig_ML_EM > 0 )
    disp('EM is preferable to ML in terms of  MSE definiteness')
else
    disp('ML and EM are not comparable in terms of  MSE definiteness')
end

if all( eig_ML_BAY < 0 )
    disp('ML is preferable to Bayeian in terms of  MSE definiteness')
elseif all( eig_ML_BAY > 0 )
    disp('Bayesian is preferable to ML in terms of  MSE definiteness')
else
    disp('ML and Bayesian are not comparable in terms of  MSE definiteness')
end

if all( eig_EM_BAY < 0 )
    disp('EM is preferable to Bayesian in terms of  MSE definiteness')
elseif all( eig_EM_BAY > 0 )
    disp('Bayesian is preferable to EM in terms of  MSE definiteness')
else
    disp('EM and Bayesian are not comparable in terms of  MSE definiteness')
end

%--------------
% Compare sigma
%--------------
sigma_true = vech(sigma);
sigma_diff_ML = sigma_ML - sigma_true(:,ones(nMC,1));
MSE_sigma_ML = sigma_diff_ML * sigma_diff_ML' / nMC;
QUAD_sigma_ML = mean( sum (sigma_diff_ML.^2) );

sigma_diff_EM = sigma_EM - sigma_true(:,ones(nMC,1));
MSE_sigma_EM = sigma_diff_EM * sigma_diff_EM' / nMC;
QUAD_sigma_EM = mean( sum (sigma_diff_EM.^2) );

sigma_diff_BAY = sigma_BAY - sigma_true(:,ones(nMC,1));
MSE_sigma_BAY = sigma_diff_BAY * sigma_diff_BAY' / nMC;
QUAD_sigma_BAY = mean( sum (sigma_diff_BAY.^2) );

disp(' ')
disp(['Quadric risk of  ML :  ',num2str(QUAD_sigma_ML)])
disp(['Quadric risk of  EM :  ',num2str(QUAD_sigma_EM)])
disp(['Quadric risk of  BAY:  ',num2str(QUAD_sigma_BAY)])

eig_ML_EM = eig(MSE_sigma_ML - MSE_sigma_EM);
eig_ML_BAY = eig(MSE_sigma_ML - MSE_sigma_BAY);
eig_EM_BAY = eig(MSE_sigma_EM - MSE_sigma_BAY);

if all( eig_ML_EM < 0 )
    disp('ML is preferable to EM in terms of  MSE definiteness')
elseif all( eig_ML_EM > 0 )
    disp('EM is preferable to ML in terms of  MSE definiteness')
else
    disp('ML and EM are not comparable in terms of  MSE definiteness')
end

if all( eig_ML_BAY < 0 )
    disp('ML is preferable to Bayeian in terms of  MSE definiteness')
elseif all( eig_ML_BAY > 0 )
    disp('Bayesian is preferable to ML in terms of  MSE definiteness')
else
    disp('ML and Bayesian are not comparable in terms of  MSE definiteness')
end

if all( eig_EM_BAY < 0 )
    disp('EM is preferable to Bayesian in terms of  MSE definiteness')
elseif all( eig_EM_BAY > 0 )
    disp('Bayesian is preferable to EM in terms of  MSE definiteness')
else
    disp('EM and Bayesian are not comparable in terms of  MSE definiteness')
end

disp(' ')
disp(['Average computation time of ML :  ',num2str(mean(time_ML))])
disp(['Average computation time of EM :  ',num2str(mean(time_EM))])
disp(['Average computation time of BAY :  ',num2str(mean(time_BAY))])

%----Comparsion Done !--------

% save results 
save results_4D.mat

% write to EXCEL file 

% Page 1: summary statistics
% heading
row_heading = {'ML','ML_STD','ML_MSE','EM','EM_STD','EM_MSE','BAY','BAY_STD','BAY_MSE'};
col_heading = cell(npara+2,1);
k = 1;
for m = 1:dim
    for n = 1:3
        col_heading{k} = ['',num2str(m),num2str(n)];
        k = k + 1;
    end
end
k = k + 1;
for m = 1:dim
    for n = m:dim
        col_heading{k} = ['',num2str(n),num2str(m)];
        k = k + 1;
    end
end
k = k + 1;
col_heading{k} = 'DF';

xlswrite('Sim results_4D.xls',row_heading,'Summary','B1')
xlswrite('Sim results_4D.xls',col_heading,'Summary','A2')
xlswrite('Sim results_4D.xls',stat_pool,'Summary','B2')


% Page 2-4: simulation details
% headings
xlswrite('Sim results_4D.xls',startMC:nMC,'ML',[native2unicode(65+startMC),'1'])
xlswrite('Sim results_4D.xls',startMC:nMC,'EM',[native2unicode(65+startMC),'1'])
xlswrite('Sim results_4D.xls',startMC:nMC,'BAY',[native2unicode(65+startMC),'1'])

xlswrite('Sim results_4D.xls',col_heading,'ML','A2')
xlswrite('Sim results_4D.xls',col_heading,'EM','A2')
xlswrite('Sim results_4D.xls',col_heading,'BAY','A2')

% write contents
xlswrite('Sim results_4D.xls',para_ML(:,startMC:nMC),'ML',[native2unicode(65+startMC),'2'])
xlswrite('Sim results_4D.xls',para_EM(:,startMC:nMC),'EM',[native2unicode(65+startMC),'2'])
xlswrite('Sim results_4D.xls',para_BAY(:,startMC:nMC),'BAY',[native2unicode(65+startMC),'2'])

% Page 5: miscellaneous
xlswrite('Sim results_4D.xls',startMC:nMC,'MISC',[native2unicode(65+startMC),'1'])
xlswrite('Sim results_4D.xls',{'time_ML';'time_EM';'time_BAY'},'MISC','A2')
xlswrite('Sim results_4D.xls',[time_ML(:,startMC:nMC);time_EM(:,startMC:nMC);time_BAY(:,startMC:nMC)],'MISC',[native2unicode(65+startMC),'2'])

xlswrite('Sim results_4D.xls',{'censor rate'},'MISC','A6')
xlswrite('Sim results_4D.xls',(0:dim)','MISC','A7')
xlswrite('Sim results_4D.xls',censor_ratio(:,startMC:nMC),'MISC',[native2unicode(65+startMC),'7'])
