clear all
tic

dim = 4;
nobs = [500,1000,2000,5000,10000,20000];
nMC = length(nobs);
df = 6;

% Preallocate memory
beta_ML = zeros(dim*3,nMC);
beta_EM = zeros(dim*3,nMC);
beta_BAY = zeros(dim*3,nMC);
beta_std_ML = zeros(dim*3,nMC);
beta_std_EM = zeros(dim*3,nMC);
beta_std_BAY = zeros(dim*3,nMC);

sigma_ML = zeros(dim*(dim+1)/2,nMC);
sigma_EM = zeros(dim*(dim+1)/2,nMC);
sigma_BAY = zeros(dim*(dim+1)/2,nMC);
sigma_std_ML = zeros(dim*(dim+1)/2,nMC);
sigma_std_EM = zeros(dim*(dim+1)/2,nMC);
sigma_std_BAY = zeros(dim*(dim+1)/2,nMC);

df_ML = zeros(1,nMC);
df_EM = zeros(1,nMC);
df_BAY = zeros(1,nMC);
df_std_ML = zeros(1,nMC);
df_std_EM = zeros(1,nMC);
df_std_BAY = zeros(1,nMC);

time_ML = zeros(1,nMC);
time_EM = zeros(1,nMC);
time_BAY = zeros(1,nMC);

% Simulation begins
H_status_bar = waitbar(0,'Simulation  ');
for r = 1:nMC
    
    waitbar(r/nMC,H_status_bar,['Simulation   ',num2str(r)]);
    
    % Generating data
    rand('seed',r*12345);
    randn('seed',r*1234567);
    sigma = 0.3*ones(dim,dim) + 0.7*eye(dim);
    epsilon = mvtrnd(sigma,df,nobs(r));    
    for m = 1:dim
        X = [ones(nobs(r),1),randn(nobs(r),1),randn(nobs(r),1)];        
        Y = X * [m;m;m] + epsilon(:,m);
        Y(Y<0) = 0;        
        eval(['X',num2str(m),'= X;'])
        eval(['Y',num2str(m),'= Y;'])
    end
    
        
    % Estimation via ML,EM,Bayesian
    if dim == 4
        try            
            tic
            waitbar(r/nMC,H_status_bar,['Simulation   ',num2str(r),' : ML in progress']);
            [beta_ML(:,r),sigma_temp,df_ML(r),beta_std_ML(:,r),sigma_std_temp,df_std_ML(r)]...
                = ML_CT_SUR(Y1,X1,Y2,X2,Y3,X3,Y4,X4);
            sigma_ML(:,r) = vech(sigma_temp);
            sigma_std_ML(:,r) = vech(sigma_std_temp);
            time_ML(r) = toc;
        catch
            disp(['Simulation ',num2str(r),': an error occurs in ML'])
        end
        
       
        try
            tic
            waitbar(r/nMC,H_status_bar,['Simulation   ',num2str(r),' : EM in progress']);
            [beta_EM(:,r),sigma_temp,df_EM(r),beta_std_EM(:,r),sigma_std_temp,df_std_EM(r)]...
                = EM_CT_SUR(Y1,X1,Y2,X2,Y3,X3,Y4,X4);
            sigma_EM(:,r) = vech(sigma_temp);
            sigma_std_EM(:,r) = vech(sigma_std_temp);
            time_EM(r) = toc;
        catch
            disp(['Simulation ',num2str(r),': an error occurs in EM '])
        end
        
        
        try
            tic
            waitbar(r/nMC,H_status_bar,['Simulation   ',num2str(r),' : Bayesian in progress']);
            [beta_BAY(:,r),sigma_temp,df_BAY(r),beta_std_BAY(:,r),sigma_std_temp,df_std_BAY(r)]...
                = GIBBS_CT_SUR(Y1,X1,Y2,X2,Y3,X3,Y4,X4);
            sigma_BAY(:,r) = vech(sigma_temp);
            sigma_std_BAY(:,r) = vech(sigma_std_temp);
            time_BAY(r) = toc;
        catch
            disp(['Simulation ',num2str(r),': an error occurs in Bayesian MCMC '])
        end
        
    else
        error('Please modify the inputs: add/delete some Yi, Xi')
    end
    
end
close(H_status_bar);


%----Simulation Done!--------------


% calculate statistics
para_ML = [beta_ML;nan(1,nMC);sigma_ML;nan(1,nMC);df_ML];
para_EM = [beta_EM;nan(1,nMC);sigma_EM;nan(1,nMC);df_EM];
para_BAY = [beta_BAY;nan(1,nMC);sigma_BAY;nan(1,nMC);df_BAY];

para_std_ML = [beta_std_ML;nan(1,nMC);sigma_std_ML;nan(1,nMC);df_std_ML];
para_std_EM = [beta_std_EM;nan(1,nMC);sigma_std_EM;nan(1,nMC);df_std_EM];
para_std_BAY = [beta_std_BAY;nan(1,nMC);sigma_std_BAY;nan(1,nMC);df_std_BAY];

npara = length(para_ML(:,1));

% save results 
save results_size.mat


disp('Writing to EXCEL file, please wait...')

% prepare the heading of EXCEL files
row_heading = nobs;
col_heading = cell(npara*2,1);
k = 1;
for m = 1:dim
    for n = 1:3
        col_heading{k} = ['',num2str(m),num2str(n)];
        k = k + 2;
    end
end
k = k + 2;
for m = 1:dim
    for n = m:dim
        col_heading{k} = ['',num2str(n),num2str(m)];
        k = k + 2;
    end
end
k = k + 2;
col_heading{k} = 'DF';

% Page 1: summary of ML etimators
k = 1;
results = cell(npara*2,nMC);
for m =1:npara
    results(k,:) = num2cell( para_ML(m,:) );
    k = k + 1;
    for n = 1:nMC
        results{k,n} = ['[ ',num2str(para_std_ML(m,n),'%4.3f'),' ]'];
    end
    k = k + 1;
end
xlswrite('Sim results_size.xls',row_heading,'ML','B1')
xlswrite('Sim results_size.xls',col_heading,'ML','A2')
xlswrite('Sim results_size.xls',results,'ML','B2')


% Page 2: summary of EM etimators
k = 1;
results = cell(npara*2,nMC);
for m =1:npara
    results(k,:) = num2cell( para_EM(m,:) );
    k = k + 1;
    for n = 1:nMC
        results{k,n} = ['[ ',num2str(para_std_EM(m,n),'%4.3f'),' ]'];
    end
    k = k + 1;
end
xlswrite('Sim results_size.xls',row_heading,'EM','B1')
xlswrite('Sim results_size.xls',col_heading,'EM','A2')
xlswrite('Sim results_size.xls',results,'EM','B2')

% Page 3: summary of BAY etimators
k = 1;
results = cell(npara*2,nMC);
for m =1:npara
    results(k,:) = num2cell( para_BAY(m,:) );
    k = k + 1;
    for n = 1:nMC
        results{k,n} = ['[ ',num2str(para_std_BAY(m,n),'%4.3f'),' ]'];
    end
    k = k + 1;
end
xlswrite('Sim results_size.xls',row_heading,'BAY','B1')
xlswrite('Sim results_size.xls',col_heading,'BAY','A2')
xlswrite('Sim results_size.xls',results,'BAY','B2')

para_ML = [beta_ML;sigma_ML;df_ML];
para_EM = [beta_EM;sigma_EM;df_EM];
para_BAY = [beta_BAY;sigma_BAY;df_BAY];
para_true = [reshape( repmat(1:dim,3,1), dim*3,1);vech(sigma);df];
QUAD_ML =  mean( (para_ML - para_true(:,ones(nMC,1))).^2 );
QUAD_EM =  mean( (para_EM - para_true(:,ones(nMC,1))).^2 ); 
QUAD_BAY =  mean( (para_BAY - para_true(:,ones(nMC,1))).^2);

% Page 4: miscellaneous
xlswrite('Sim results_size.xls',row_heading,'MISC','B1')
xlswrite('Sim results_size.xls',{'time_ML';'time_EM';'time_BAY'},'MISC','A2')
xlswrite('Sim results_size.xls',[time_ML;time_EM;time_BAY],'MISC','B2')

xlswrite('Sim results_size.xls',{'Quad_risk_ML';'Quad_risk_EM';'Quad_risk_BAY'},'MISC','A7')
xlswrite('Sim results_size.xls',[QUAD_ML;QUAD_EM;QUAD_BAY],'MISC','B7')

disp('Sim results_size.xls has been successfully prepared ')